// #############################################################################
// # lib_goborotate.c - Gobo wheel library for 4 coil unipolar stepper motors, #
// #                    controled to make half steps                           # 
// #############################################################################
// # Version: 1.0 - Compiler: AVR-GCC 4.3.3 (Linux)                            #
// #  (c) 2009 by Malte Pöggel - www.MALTEPOEGGEL.de - malte@maltepoeggel.de   #
// #############################################################################
// #  This program is free software; you can redistribute it and/or modify it  #
// #   under the terms of the GNU General Public License as published by the   #
// #        Free Software Foundation; either version 3 of the License,         #
// #                  or (at your option) any later version.                   #
// #                                                                           #
// #      This program is distributed in the hope that it will be useful,      #
// #      but WITHOUT ANY WARRANTY; without even the implied warranty of       #
// #           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.            #
// #           See the GNU General Public License for more details.            #
// #                                                                           #
// #  You should have received a copy of the GNU General Public License along  #
// #      with this program; if not, see <http://www.gnu.org/licenses/>.       #
// #############################################################################

#include "lib_goborotate.h"

volatile uint16_t Ist = 0;
volatile uint16_t Soll = 0;
volatile uint8_t Step = 0;

// Private Functions
void SetPort( uint8_t value );
void MakeStep( uint8_t direction );

// Initialize Ports and set wheel to zero position
extern void Init_Gobo( void )
 {
  int i;
  DDR_COIL1 |= (1<<PIN_COIL1);
  DDR_COIL2 |= (1<<PIN_COIL2);
  DDR_COIL3 |= (1<<PIN_COIL3);
  DDR_COIL4 |= (1<<PIN_COIL4);
  DDR_SENSOR &= ~(1<<PIN_SENSOR);

  for(i=0; i<=8; i++)				// Make shure that motor position fits
   {
    MakeStep(1);
   }
  for(i=0; i<=(MOT_STEPS/2); i++)		// If sensor pin is pulled low, turn the gobowheel
   {						// until sensor is out of zero position.
    if((PR_SENSOR & (1<<PIN_SENSOR))) break;
    MakeStep(1);
    MakeStep(1);
    _delay_ms(4);
   }
  for(i=0; i<=MOT_STEPS; i++)			// Now find zero position again!
   {
    MakeStep(0);
    MakeStep(0);
    _delay_ms(4);
    if(!(PR_SENSOR & (1<<PIN_SENSOR))) break;
   }
  for(i=0; i<=CORR_VAL; i++)			// Correct position with fixed value if needed
   {
    MakeStep(CORR_DIR);
   }
 }


// Select gobo position
extern void SetGobo( uint8_t gobo )
 {
  Soll = gobo*GOBO_HALFSTEPS;
 }


/*
// This would rotate only in one direction
void CheckGobo( void )
 {
  if(Ist!=Soll) 
   {
    MakeStep(0);
    Ist++;
    if(Ist>=MOT_STEPS) Ist=0; 
   }
 }
*/


// Check if the wheel needs to be rotated
extern void CheckGobo( void )
 {
  if(Ist<Soll) 
   {
    if((Soll-Ist)<=(MOT_STEPS/2))	// We assumed the right direction because
     {					// its fewer or equal to an half rotation
      MakeStep(0);			// Spin it right round =)
      Ist++;
      if(Ist>=MOT_STEPS) Ist=0;
     } else {				// Right rotation would take to much time so
      MakeStep(1);			// rotate the wheel left direction
      if(Ist==0) Ist=MOT_STEPS;
      Ist--;
     }
   } 
  if(Ist>Soll) 
   {
    if((Ist-Soll)<=(MOT_STEPS/2))
     {
      MakeStep(1);
      if(Ist==0) Ist=MOT_STEPS; 
      Ist--;     
     } else {
      MakeStep(0);
      Ist++;
      if(Ist>=MOT_STEPS) Ist=0;  
     }
   }
 }


// Step the motor in defined direction
void MakeStep( uint8_t direction )
 {
  if(direction==0) 
   {
    Step++;
    if(Step>=8) Step=0;
   } else {
    if(Step==0) Step=8;
    Step--;
   }
  SetPort(Step);
  _delay_ms(4);
 }


// Switch the ports
void SetPort( uint8_t value )
 {
  switch(value) 
   {
    case 0:
     PRT_COIL1 |=  (1<<PIN_COIL1);
     PRT_COIL2 &= ~(1<<PIN_COIL2);
     PRT_COIL3 &= ~(1<<PIN_COIL3);
     PRT_COIL4 &= ~(1<<PIN_COIL4);
     break; 
    case 1:
     PRT_COIL1 |=  (1<<PIN_COIL1);
     PRT_COIL2 |=  (1<<PIN_COIL2);
     PRT_COIL3 &= ~(1<<PIN_COIL3);
     PRT_COIL4 &= ~(1<<PIN_COIL4);
     break;    
    case 2:
     PRT_COIL1 &= ~(1<<PIN_COIL1);
     PRT_COIL2 |=  (1<<PIN_COIL2);
     PRT_COIL3 &= ~(1<<PIN_COIL3);
     PRT_COIL4 &= ~(1<<PIN_COIL4);
     break;
    case 3:
     PRT_COIL1 &= ~(1<<PIN_COIL1);
     PRT_COIL2 |=  (1<<PIN_COIL2);
     PRT_COIL3 |=  (1<<PIN_COIL3);
     PRT_COIL4 &= ~(1<<PIN_COIL4);
     break; 
    case 4:
     PRT_COIL1 &= ~(1<<PIN_COIL1);
     PRT_COIL2 &= ~(1<<PIN_COIL2);
     PRT_COIL3 |=  (1<<PIN_COIL3);
     PRT_COIL4 &= ~(1<<PIN_COIL4);
     break;
    case 5:
     PRT_COIL1 &= ~(1<<PIN_COIL1);
     PRT_COIL2 &= ~(1<<PIN_COIL2);
     PRT_COIL3 |=  (1<<PIN_COIL3);
     PRT_COIL4 |=  (1<<PIN_COIL4);
     break; 
    case 6:
     PRT_COIL1 &= ~(1<<PIN_COIL1);
     PRT_COIL2 &= ~(1<<PIN_COIL2);
     PRT_COIL3 &= ~(1<<PIN_COIL3);
     PRT_COIL4 |=  (1<<PIN_COIL4);
     break;
    case 7:
     PRT_COIL1 |=  (1<<PIN_COIL1);
     PRT_COIL2 &= ~(1<<PIN_COIL2);
     PRT_COIL3 &= ~(1<<PIN_COIL3);
     PRT_COIL4 |=  (1<<PIN_COIL4);
     break; 
   }
 }

